
package org.jbpm.groupNotification;

import java.util.List;

import org.jbpm.api.ProcessInstance;
import org.jbpm.api.task.Task;
import org.jbpm.test.JbpmTestCase;
import org.subethamail.wiser.Wiser;
import org.subethamail.wiser.WiserMessage;


public class GroupNotificationTest extends JbpmTestCase {

  String deploymentId;
  String groupNotify;
  Wiser wiser = new Wiser();	   

  protected void setUp() throws Exception {
    super.setUp();

    // create identities
    groupNotify = identityService.createGroup("groupNotify");

    identityService.createUser("johndoe", "John", "Doe");
    identityService.createMembership("johndoe", groupNotify, "developer");

    identityService.createUser("joesmoe", "Joe", "Smoe");
    identityService.createMembership("joesmoe", groupNotify, "developer");
    
	// start mail server
	wiser.setPort(2525);
	wiser.start();
	
    // deploy process
    deploymentId = repositoryService.createDeployment()
        .addResourceFromClasspath("groupNotification.jpdl.xml")
        .deploy();
  }

  protected void tearDown() throws Exception {
	  
	// stop mail server
	wiser.stop();		  
    // delete process deployment
    repositoryService.deleteDeploymentCascade(deploymentId);

    // delete identities
    identityService.deleteGroup(groupNotify);
    identityService.deleteUser("johndoe");
    identityService.deleteUser("joesmoe");

    super.tearDown();
  }

  public void testProcess() {

    ProcessInstance processInstance = executionService.startProcessInstanceByKey("groupNotification");
    String pid = processInstance.getId();

	 // examine produced messages
	 List<WiserMessage> wiserMessages = wiser.getMessages();
	 
	 //johndoe, joeshmoe
	 assertEquals(2, wiserMessages.size());
	 
    // both johndoe and joesmoe will see the task in their *group* task list 
    List<Task> taskList = taskService.findGroupTasks("johndoe");
    assertEquals("Expected a single task in johndoe's task list", 1, taskList.size());
    Task task = taskList.get(0);
    assertEquals("review", task.getName());

    taskList = taskService.findGroupTasks("joesmoe");
    assertEquals("Expected a single task in joesmoe's group task list", 1, taskList.size());
    task = taskList.get(0);
    assertEquals("review", task.getName());

    // johndoe and joesmoe will NOT see the task in their *assigned* task list 

    taskList = taskService.findPersonalTasks("johndoe");
    assertEquals(0, taskList.size());

    taskList = taskService.findPersonalTasks("joesmoe");
    assertEquals(0, taskList.size());

    // lets assume that johndoe takes the task
    taskService.takeTask(task.getId(), "johndoe");

    // johndoe's and joesmoe's group task list is now empty  
    taskList = taskService.findGroupTasks("johndoe");
    assertEquals(0, taskList.size());

    taskList = taskService.findGroupTasks("joesmoe");
    assertEquals(0, taskList.size());

    // johndoe's assigned task list has the task in it
    taskList = taskService.findPersonalTasks("johndoe");
    assertEquals("Expected a single task being created", 1, taskList.size());
    task = taskList.get(0);
    assertEquals("review", task.getName());
    assertEquals("johndoe", task.getAssignee());

    // joesmoe will not see the task in the assigned task list
    taskList = taskService.findPersonalTasks("joesmoe");
    assertEquals(0, taskList.size());

    // now johndoe completes the task
    taskService.completeTask(task.getId());

    // verify that the task list is now empty
    taskList = taskService.findPersonalTasks("johndoe");
    assertEquals(0, taskList.size());

    // verify that process moved to the next state
    processInstance = executionService.findProcessInstanceById(pid);
    assertTrue(processInstance.isActive("wait"));
  }
}
